# -*- coding: utf-8 -*-
# Licensed under the MIT license:
# Copyright (c) 2013 Bit Shift <bitshift@bigmacintosh.net>
# Copyright (c) 2016 Pol Van Aubel <dev@polvanaubel.com>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:

# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# http://www.opensource.org/licenses/mit-license.php
#
# Revision log:
# 0.3   Pol Van Aubel <dev@polvanaubel.com>
#       Make Python3 compatible.
#       Feed fetching should now timeout after 15 seconds.
#       Handle negative return values correctly.
#       Fix deleting from active iteration object in unload.
#
# 0.2.3 Pol Van Aubel <dev@polvanaubel.com>
#       Changed weechat.prnt to weechat.prnt_date_tags where messages
#       are not a direct result from user input; errors get tagged with
#       irc_error and notify_message, messages get tagged with
#       notify_message.
#
# 0.2.2 Version from Bit Shift

import weechat
import string
import feedparser

import sys # Only required for python version check.
PY2 = sys.version_info < (3,)

weechat.register(
        "weemustfeed",
        "Bit Shift <bitshift@bigmacintosh.net>",
        "0.3",
        "MIT",
        "RSS/Atom/RDF aggregator for weechat",
        "",
        ""
        )


default_settings = {
        "interval": "300",
        "feeds": ""
        }

weemustfeed_buffer = None
weemustfeed_timer = None
fetch_hooks = {}
updating = set()
partial_feeds = {}

help_message = """
COMMANDS:
a <name> <url>        Add a feed with display name of <name> and URL of <url>.
d <name>              Delete the feed with display name <name>.
u <name> <url>        Update the feed with display name <name> to use URL <url>.
l                     List all feeds known to WeeMustFeed.
t <name>              Toggle a feed - disable/enable it temporarily without fully removing it.
?                     Display this help message.

CONFIG:
plugins.var.python.weemustfeed.interval
    Interval between update checks, in seconds. Must be a number, but is stored
    as a string. Blame the scripting API.
    default: "300"
""".strip()


def show_help():
    for line in help_message.split("\n"):
        weechat.prnt(weemustfeed_buffer, "\t\t" + line)


def weemustfeed_input_cb(data, buffer, input_data):
    global updating

    chunks = input_data.split()

    if chunks[0] == "a":
        if len(chunks) != 3:
            weechat.prnt(weemustfeed_buffer, weechat.prefix("error") + "Wrong number of parameters. Syntax is 'a <name> <url>'.")
            return weechat.WEECHAT_RC_ERROR
        elif any([c not in (string.ascii_letters + string.digits) for c in chunks[1]]):
            weechat.prnt(weemustfeed_buffer, weechat.prefix("error") + "Only A-Z, a-z, and 0-9 permitted in names.")
            return weechat.WEECHAT_RC_ERROR
        else:
            current_feeds = weechat.config_get_plugin("feeds").strip().split(";")

            if chunks[1] in current_feeds:
                weechat.prnt(weemustfeed_buffer, weechat.prefix("error") + "A feed with that name already exists (note: feed names are case-insensitive).")
                return weechat.WEECHAT_RC_ERROR
            else:
                current_feeds.append(chunks[1])
                weechat.config_set_plugin("feed." + chunks[1].lower() + ".url", chunks[2])
                weechat.config_set_plugin("feeds", ";".join(current_feeds))
                weechat.prnt(weemustfeed_buffer, "Added '" + chunks[1] + "'.")
    elif chunks[0] == "d":
        if len(chunks) != 2:
            weechat.prnt(weemustfeed_buffer, weechat.prefix("error") + "Wrong number of parameters. Syntax is 'd <name>'.")
            return weechat.WEECHAT_RC_ERROR
        elif any([c not in (string.ascii_letters + string.digits) for c in chunks[1]]):
            weechat.prnt(weemustfeed_buffer, weechat.prefix("error") + "Only A-Z, a-z, and 0-9 permitted in names.")
            return weechat.WEECHAT_RC_ERROR
        else:
            current_feeds = weechat.config_get_plugin("feeds").strip().split(";")
            if not chunks[1] in current_feeds:
                weechat.prnt(weemustfeed_buffer, weechat.prefix("error") + "No such feed exists.")
                return weechat.WEECHAT_RC_ERROR
            else:
                current_feeds.remove(chunks[1])
                weechat.config_set_plugin("feeds", ";".join(current_feeds))
                weechat.prnt(weemustfeed_buffer, "Deleted '" + chunks[1] + "'.")
    elif chunks[0] == "u":
        if len(chunks) != 3:
            weechat.prnt(weemustfeed_buffer, weechat.prefix("error") + "Wrong number of parameters. Syntax is 'u <name> <url>'.")
            return weechat.WEECHAT_RC_ERROR
        elif any([c not in (string.ascii_letters + string.digits) for c in chunks[1]]):
            weechat.prnt(weemustfeed_buffer, weechat.prefix("error") + "Only A-Z, a-z, and 0-9 permitted in names.")
            return weechat.WEECHAT_RC_ERROR
        else:
            current_feeds = weechat.config_get_plugin("feeds").strip().split(";")

            if not chunks[1] in current_feeds:
                weechat.prnt(weemustfeed_buffer, weechat.prefix("error") + "No feed with that name currently exists (note: feed names are case-insensitive).")
                return weechat.WEECHAT_RC_ERROR
            else:
                weechat.config_set_plugin("feed." + chunks[1].lower() + ".url", chunks[2])
                weechat.config_set_plugin("feeds", ";".join(current_feeds))
                weechat.prnt(weemustfeed_buffer, "Updated '" + chunks[1] + "'.")
    elif chunks[0] == "l":
        if len(chunks) != 1:
            weechat.prnt(weemustfeed_buffer, weechat.prefix("error") + "Wrong number of parameters. Syntax is 'l'.")
            return weechat.WEECHAT_RC_ERROR
        else:
            current_feeds = weechat.config_get_plugin("feeds").strip().split(";")
            for feed in current_feeds:
                if feed != "":
                    if (weechat.config_is_set_plugin("feed." + feed.lower() + ".enabled") and
                        weechat.config_get_plugin("feed." + feed.lower() + ".enabled").lower() != "yes"):
                        feed_status = "disabled"
                    elif not (weechat.config_is_set_plugin("feed." + feed.lower() + ".last_id") and
                              weechat.config_get_plugin("feed." + feed.lower() + ".last_id") != ""):
                        feed_status = "new"
                    elif feed in updating:
                        feed_status = "updating"
                    elif (weechat.config_is_set_plugin("feed." + feed.lower() + ".enabled") and
                          weechat.config_get_plugin("feed." + feed.lower() + ".enabled").lower() != "yes"):
                        feed_status = "disabled"
                    else:
                        feed_status = "enabled"
                    weechat.prnt(weemustfeed_buffer, "\t" + feed + ": " + weechat.config_get_plugin("feed." + feed.lower() + ".url") + " [" + feed_status + "]")
    elif chunks[0] == "t":
        if len(chunks) != 2:
            weechat.prnt(weemustfeed_buffer, weechat.prefix("error") + "Wrong number of parameters. Syntax is 't <name>'.")
            return weechat.WEECHAT_RC_ERROR
        elif any([c not in (string.ascii_letters + string.digits) for c in chunks[1]]):
            weechat.prnt(weemustfeed_buffer, weechat.prefix("error") + "Only A-Z, a-z, and 0-9 permitted in names.")
            return weechat.WEECHAT_RC_ERROR
        else:
            current_feeds = weechat.config_get_plugin("feeds").strip().split(";")
            if not chunks[1] in current_feeds:
                weechat.prnt(weemustfeed_buffer, weechat.prefix("error") + "No such feed exists.")
                return weechat.WEECHAT_RC_ERROR
            else:
                if not weechat.config_is_set_plugin("feed." + chunks[1].lower() + ".enabled"):
                    feed_enabled = True
                else:
                    feed_enabled = (weechat.config_get_plugin("feed." +
                        chunks[1].lower() + ".enabled").lower() == "yes")

                if feed_enabled:
                    weechat.config_set_plugin("feed." + chunks[1].lower() +
                            ".enabled", "no")
                    weechat.prnt(weemustfeed_buffer, "Disabled '" + chunks[1] + "'.")
                else:
                    weechat.config_set_plugin("feed." + chunks[1].lower() +
                            ".enabled", "yes")
                    weechat.prnt(weemustfeed_buffer, "Enabled '" + chunks[1] + "'.")
    elif chunks[0] == "?":
        if len(chunks) != 1:
            weechat.prnt(weemustfeed_buffer, weechat.prefix("error") + "Wrong number of parameters. Syntax is '?'.")
            return weechat.WEECHAT_RC_ERROR
        else:
            show_help()

    return weechat.WEECHAT_RC_OK


def weemustfeed_close_cb(data, buffer):
    global weemustfeed_buffer, weemustfeed_timer

    weemustfeed_buffer = None
    weechat.unhook(weemustfeed_timer)
    for feed in list(fetch_hooks):
        weechat.unhook(fetch_hooks[feed])
        del fetch_hooks[feed]
    weemustfeed_timer = None
    return weechat.WEECHAT_RC_OK


def weemustfeed_command_cb(data, buffer, args):
    global weemustfeed_buffer

    if weemustfeed_buffer is None:
        weemustfeed_buffer = weechat.buffer_new(
                "weemustfeed",
                "weemustfeed_input_cb", "",
                "weemustfeed_close_cb", ""
                )

        weechat.buffer_set(weemustfeed_buffer, "title",
                "WeeMustFeed - a: Add feed, d: Delete feed, u: Update URL, l: List feeds, t: Toggle feed, ?: Show help")

        set_timer()

    weechat.buffer_set(weemustfeed_buffer, "display", "1") # switch to it

    return weechat.WEECHAT_RC_OK


def weemustfeed_reset_timer_cb(data, option, value):
    if weemustfeed_timer is not None:
        unset_timer()
        set_timer()
    return weechat.WEECHAT_RC_OK


def weemustfeed_update_single_feed_cb(feed, command, return_code, out, err):
    global partial_feeds, updating

    if not feed in partial_feeds:
        partial_feeds[feed] = ""

    if return_code == weechat.WEECHAT_HOOK_PROCESS_RUNNING:  # feed not done yet
        partial_feeds[feed] += out
        return weechat.WEECHAT_RC_OK
    elif return_code == weechat.WEECHAT_HOOK_PROCESS_ERROR:
        weechat.prnt_date_tags(weemustfeed_buffer, 0,
                "irc_error,notify_message",
                weechat.prefix("error") + "Hook process error for feed '" + feed + "'. === " + command + " === " + out + " === " + err)
        status = weechat.WEECHAT_RC_ERROR
    elif return_code == 1:
        weechat.prnt_date_tags(weemustfeed_buffer, 0,
                "irc_error,notify_message",
                weechat.prefix("error") + "Invalid URL for feed '" + feed + "'.")
        status = weechat.WEECHAT_RC_ERROR
    elif return_code == 2:
        weechat.prnt_date_tags(weemustfeed_buffer, 0,
                "irc_error,notify_message",
                weechat.prefix("error") + "Transfer error while fetching feed '" + feed + "'.")
        status = weechat.WEECHAT_RC_ERROR
    elif return_code == 3:
        weechat.prnt_date_tags(weemustfeed_buffer, 0,
                "irc_error,notify_message",
                weechat.prefix("error") + "Out of memory while fetching feed '" + feed + "'.")
        status = weechat.WEECHAT_RC_ERROR
    elif return_code == 4:
        weechat.prnt_date_tags(weemustfeed_buffer, 0,
                "irc_error,notify_message",
                weechat.prefix("error") + "Error with a file while fetching feed '" + feed + "'.")
        status = weechat.WEECHAT_RC_ERROR
    elif return_code == 0:  # all good, and we have a complete feed
        if not weechat.config_is_set_plugin("feed." + feed.lower() + ".last_id"):
            weechat.config_set_plugin("feed." + feed.lower() + ".last_id", "")
            last_id = ""

        last_id = weechat.config_get_plugin("feed." + feed.lower() + ".last_id")

        parsed_feed = feedparser.parse(partial_feeds[feed] + out)

        entries = list(reversed(parsed_feed.entries))

        for entry in entries:
            if not hasattr(entry, "id"):
                entry.id = entry.link

        if (last_id == "") and len(entries) > 0:
            last_id = entries[-1].id
        else:
            if last_id in [entry.id for entry in entries]:
                only_new = False
            else:
                only_new = True

            for entry in entries:
                if PY2:
                    entrytitle = entry.title.encode("utf-8")
                    entryurl = entry.link.encode("utf-8")
                else:
                    entrytitle = entry.title
                    entryurl = entry.link

                if only_new:
                    weechat.prnt_date_tags(weemustfeed_buffer, 0, "notify_message", "{feed}\t{title} {url}".format(**{
                        "feed": feed,
                        "title": entrytitle,
                        "url": entryurl
                        }))
                    last_id = entry.id
                elif entry.id == last_id:
                    only_new = True  # everything else will be newer

        weechat.config_set_plugin("feed." + feed.lower() + ".last_id", last_id)

        status = weechat.WEECHAT_RC_OK

    else: # Unknown return code. Script must be updated.
        weechat.prnt_date_tags(weemustfeed_buffer, 0,
                "irc_error,notify_message",
                weechat.prefix("error") + "Unknown return code " + return_code + " for feed '" + feed + "'. Script must be updated.")
        status = weechat.WEECHAT_RC_ERROR

    partial_feeds[feed] = ""
    if feed in updating:
        updating.remove(feed)
    if feed in fetch_hooks:
        del fetch_hooks[feed]
    return status


def weemustfeed_update_feeds_cb(data, remaining_calls):
    global updating

    for feed in weechat.config_get_plugin("feeds").strip().split(";"):
        if weechat.config_is_set_plugin("feed." + feed.lower() + ".url"):
            if not (weechat.config_is_set_plugin("feed." + feed.lower() + ".enabled") and
                    weechat.config_get_plugin("feed." + feed.lower() + ".enabled").lower() != "yes"):
                updating.add(feed)
                if not feed in fetch_hooks:
                    fetch_hooks[feed] = weechat.hook_process(
                        "url:" + weechat.config_get_plugin("feed." + feed.lower() + ".url"),
                        15000,
                        "weemustfeed_update_single_feed_cb", feed
                        )
        elif feed != "":
            weechat.prnt_date_tags(weemustfeed_buffer, 0,
                    "irc_error,notify_message",
                    weechat.prefix("error") + "Feed '" + feed + "' has no URL set.")
    return weechat.WEECHAT_RC_OK


def set_timer():
    global weemustfeed_timer

    try:
        timer_interval = int(weechat.config_get_plugin("interval"))
    except ValueError:
        timer_interval = int(default_settings["interval"])

    weemustfeed_timer = weechat.hook_timer(
            timer_interval * 1000,
            0,
            0,
            "weemustfeed_update_feeds_cb", ""
            )


def unset_timer():
    if weemustfeed_timer is not None:
        weechat.unhook(weemustfeed_timer)


def init_script():
    global default_settings

    for option, default_value in list(default_settings.items()):
        if not weechat.config_is_set_plugin(option):
            weechat.config_set_plugin(option, default_value)

    weechat.hook_command(
        "weemustfeed",
        "open/switch to weemustfeed buffer",
        "",
        "",
        "",
        "weemustfeed_command_cb", ""
        )

    weechat.hook_config(
        "plugins.var.python.weemustfeed.interval",
        "weemustfeed_reset_timer_cb", ""
        )


init_script()



# -*- coding: utf-8 -*-
# execbot.py
# ExecBot - Executing bot
# ==============================
#
# Copyright (C) 2018 Giap Tran <txgvnn@gmail.com>
#  https://github.com/txgvnn/weechat-execbot
#
# == About =====================
# Execbot is a bot can execute the command from IRC message
# (You maybe want to call it is a kind of backdoor)
#
# With server is installed execbot script, the client can execute the
# command remote by send privmsg to server's nick in IRC network
#
# == Usage =====================
# /excebot list                                List server.nicknames are allowed
# /execbot add -server <server> nicknames..    Add server.nicknames are allowed to execute
# /execbot del -server <server> nicknames..    Remove server.nicknames are allowed
#
# == NOTICE ====================
# This is a POC that I want to execute a remote command via IRC
# network. It's really not security, although you can set who can
# talk with the bot.
#
# PLEASE CONSIDER CAREFULLY WHEN USING THE PLUGIN

import weechat

SCRIPT_NAME      = 'execbot'
SCRIPT_AUTHOR    = 'Giap Tran <txgvnn@gmail.com>'
SCRIPT_VERSION   = '1.1'
SCRIPT_LICENSE   = 'GPL3'
SCRIPT_DESC      = 'Executing bot'
SCRIPT_HELP_TEXT = '''
%(bold)sExecbot command options: %(normal)s
list                                   List nicknames
add -server <server>  <nicknames...>   Add server.nicknames are allowed to execute
del -server <server>  <nicknames...>   Del server.nicknames are allowed

%(bold)sExamples: %(normal)s
Allow john and jack in oftc server to talk with bot
  /execbot add -server oftc john jack
''' % {'bold':weechat.color('bold'), 'normal':weechat.color('-bold')}

execbot_config_file      = None
execbot_config_section   = {}
execbot_allows           = {}

def execbot_config_init():
    '''Init configuration file'''
    global execbot_config_file
    execbot_config_file = weechat.config_new('execbot', 'execbot_config_reload_cb', '')
    if not execbot_config_file:
        return

    execbot_config_section['allows'] = weechat.config_new_section(
        execbot_config_file, 'allows', 0, 0, 'execbot_config_allows_read_cb', '',
        'execbot_config_allows_write_cb', '', '', '', '', '', '', '')
    if not execbot_config_section['allows']:
        weechat.config_free(execbot_config_file)

def execbot_config_reload_cb(data, config_file):
    '''Handle a reload of the configuration file.'''
    global execbot_allows
    execbot_allows   = {}
    return weechat.config_reload(config_file)

def execbot_config_allows_read_cb(data, config_file, section_name, option_name, value):
    '''Read elements of the allows section from the configuration file.'''
    execbot_allows[option_name.lower()] = value
    return weechat.WEECHAT_CONFIG_OPTION_SET_OK_CHANGED

def execbot_config_allows_write_cb(data, config_file, section_name):
    '''Write infomation to the allows section of the configuration file.'''
    weechat.config_write_line(config_file, section_name, '')
    for username, right in sorted(list(execbot_allows.items())):
        weechat.config_write_line(config_file, username.lower(), right)
    return weechat.WEECHAT_RC_OK

def execbot_config_read():
    ''' Read Execbot configuration file (execbot.conf).'''
    return weechat.config_read(execbot_config_file)

def execbot_config_write():
    ''' Write Execbot configuration file (execbot.conf) to disk.'''
    return weechat.config_write(execbot_config_file)

def execbot_command_list():
    '''List server.nicknames are allowed.'''
    nicknames = '\n'.join([' %s' % x for x in execbot_allows.keys()])
    # for nickname,_ in sorted(list(execbot_allows.items())):
    weechat.prnt(weechat.current_buffer(), 'Nicknames are allowed:\n' +nicknames)
    return weechat.WEECHAT_RC_OK

def execbot_command_add(server,nicknames):
    '''Add nicknames.'''
    for x in nicknames:
        execbot_allows['%s.%s'%(server,x.lower())] = 'allow'
        weechat.prnt(weechat.current_buffer(),'Add permission for %s' % '%s.%s'%(server,x.lower()))
    return weechat.WEECHAT_RC_OK

def execbot_command_del(server,nicknames):
    '''Remove nicknames.'''
    for x in nicknames:
        try:
            del execbot_allows['%s.%s'%(server,x.lower())]
            weechat.prnt(weechat.current_buffer(),'Deleted permission of %s' % '%s.%s'%(server,x.lower()))
        except KeyError:
            weechat.prnt(weechat.current_buffer(),'No existing %s.%s'%(server,x.lower()))
    return weechat.WEECHAT_RC_OK

def execbot_command(data, buffer, args):
    '''Hook to handle the /execbot weechat command.'''
    argv = args.split()

    # list
    if not argv or argv == ['list']:
        return execbot_command_list()

    # check if a server was set
    if (len(argv) > 2 and argv[1] == '-server'):
        server = argv[2]
        del argv[1:3]
        args = (args.split(' ', 2)+[''])[2]
    else:
        server = weechat.buffer_get_string(buffer, 'localvar_server')
    if not server:
        weechat.prnt(weechat.current_buffer(), 'Required -server option')
        return weechat.WEECHAT_RC_ERROR

    # add
    if argv[:1] == ['add']:
        if len(argv) < 2:
            return weechat.WEECHAT_RC_ERROR
        return execbot_command_add(server,argv[1:])

    # del
    if argv[:1] == ['del']:
        if len(argv) < 2:
            return weechat.WEECHAT_RC_ERROR
        return execbot_command_del(server,argv[1:])

    execbot_error('Unknown command. Try  /help execbot', buffer)
    return weechat.WEECHAT_RC_OK

def execbot_process(buffer, command, return_code, out, err):
    '''Execute the command and return to buffer.'''
    message = "%s ... | $? = %d\n" % (command.split()[0], return_code)
    if out != "":
        message += out
    if err != "":
        message += err

    weechat.command(buffer, message)
    return weechat.WEECHAT_RC_OK

def execbot_hook_signal(data, signal, signal_data):
    server = signal.split(",")[0]
    info = weechat.info_get_hashtable("irc_message_parse", { "message": signal_data })
    username = '.'.join([server,info['nick']])

    # Check the permission
    allowed = execbot_allows.get(username.lower())
    if not allowed:
        return weechat.WEECHAT_RC_OK
    # Prevent public channel
    if info['channel'].startswith('#'):
        return weechat.WEECHAT_RC_OK

    # buffer output
    buffer = weechat.buffer_search("irc", username)
    # command
    _, command = info['arguments'].split(':', 1)

    # timeout = 5 mins
    weechat.hook_process(command, 300000, "execbot_process", buffer)

    return weechat.WEECHAT_RC_OK

def execbot_unload_script():
    execbot_config_write()
    return weechat.WEECHAT_RC_OK

if __name__ == '__main__' and weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR,
                                               SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, 'execbot_unload_script','UTF-8'):
    execbot_config_init()
    execbot_config_read()
    weechat.hook_command('execbot', 'Commands to manage Execbot options and execute Execbot commands',
                         '',
                         SCRIPT_HELP_TEXT,
                         'list %- || add -server %(irc_servers) %(nicks) %-'
                         '|| del -server %(irc_servers) %(nicks) %-',
                         'execbot_command', '')
    weechat.hook_signal("*,irc_in2_privmsg", "execbot_hook_signal", "")



#
# SPDX-FileCopyrightText: 2012-2026 nils_2 <weechatter@arcor.de>
# SPDX-FileCopyrightText: 2012-2017 nesthib <nesthib@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# scroll indicator; displaying number of lines below last line, overall lines in buffer, number of current line and percent displayed
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# 2026-05-23: nils_2 (libera.#weechat)
#        0.9: fix: two SyntaxWarning
# 2017-08-17: nils_2 (freenode.#weechat)
#        0.8: add support for buffer_filters_enabled and buffer_filters_disabled (WeeChat ≥ 2.0)
# 2016-12-16: nils_2 (freenode.#weechat)
#        0.7: add option show_scroll (idea by earnestly)
# 2016-04-23: wdbw <tuturu@tutanota.com>
#     0.6.2 : fix: type of filters_enabled
# 2014-02-24: nesthib (freenode.#weechat)
#     0.6.1 : fix: color tags for default format
# 2013-11-19: nils_2 (freenode.#weechat)
#       0.6 : fix: stdout/stderr warning
# 2013-11-02: nils_2 (freenode.#weechat)
#       0.5 : fix refresh on (un)zoomed buffer
#           : add option 'count_filtered_lines' and format item "%F"
# 2013-10-15: nils_2 (freenode.#weechat)
#       0.4 : fix bug with root-bar
#           : add support of eval_expression (weechat >= 0.4.2)
# 2013-01-25: nils_2 (freenode.#weechat)
#       0.3 : make script compatible with Python 3.x
#           : internal changes
# 2012-07-09: nils_2 (freenode.#weechat)
#       0.2 : fix: display bug with more than one window
#           : hide item when buffer empty
# 2012-07-08: obiwahn
#     0.1.1 : add hook for switch_buffer
# 2012-01-11: nils_2, nesthib (freenode.#weechat)
#       0.1 : initial release
#
# Development is currently hosted at
# https://github.com/weechatter/weechat-scripts

try:
    import weechat, re

except Exception:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    quit()

SCRIPT_NAME     = "bufsize"
SCRIPT_AUTHOR   = "nils_2 <weechatter@arcor.de>"
SCRIPT_VERSION  = "0.9"
SCRIPT_LICENSE  = "GPL"
SCRIPT_DESC     = "scroll indicator; displaying number of lines below last line, overall lines in buffer, number of current line and percent displayed"

OPTIONS         = { 'format'            : ('${color:yellow}%P${color:default}⋅%{${color:yellow}%A${color:default}⇵${color:yellow}%C${color:default}/}${color:yellow}%L',
                                           'format for items to display in bar, possible items: %P = percent indicator, %A = number of lines below last line, %L = lines counter, %C = current line %F = number of filtered lines (note: using WeeChat >= 0.4.2, content is evaluated, so you can use colors with format \"${color:xxx}\", see /help eval)'),
                    'count_filtered_lines': ('on',
                                           'filtered lines will be count in item.'),
                    'show_scroll':('on','always show the scroll indicator number,even if its 0 (item %A), if option is off the scroll indicator will be hidden like the item "scroll"'),
                   }
# ================================[ weechat item ]===============================
# regexp to match ${color} tags
regex_color=re.compile(r'\$\{([^\{\}]+)\}')

# regexp to match ${optional string} tags
regex_optional_tags=re.compile(r'%\{[^\{\}]+\}')

filter_status = 0

def show_item (data, item, window):
    # check for root input bar!
    if not window:
       window = weechat.current_window()

    ptr_buffer = weechat.window_get_pointer(window,'buffer')
    if ptr_buffer == '':
        return ''

    if weechat.buffer_get_string(ptr_buffer,'name') != 'weechat':                         # not weechat core buffer
        if weechat.buffer_get_string(ptr_buffer,'localvar_type') == '':                   # buffer with free content?
          return ''

    lines_after, lines_count, percent, current_line, filtered, filtered_before, filtered_after = count_lines(window,ptr_buffer)
    lines_after_bak = lines_after

    if lines_count == 0:                                                                  # buffer empty?
        return ''

    if filtered == 0:
        filtered = ''

    if lines_after == 0 and (OPTIONS['show_scroll'].lower() == 'off'):
        lines_after = ''

    tags = {'%C': str(current_line),
            '%A': str(lines_after),
            '%F': str(filtered),
            '%L': str(lines_count),
            '%P': str(percent)+'%'}

    bufsize_item = substitute_colors(OPTIONS['format'])

    # replace mandatory tags
    for tag in list(tags.keys()):
#    for tag in tags.keys():
        bufsize_item = bufsize_item.replace(tag, tags[tag])

    # replace optional tags
    # %{…} only if lines after (e.g. %A > 0)
    if lines_after_bak > 0:
        for regex_tag in regex_optional_tags.findall(bufsize_item):
            bufsize_item = bufsize_item.replace(regex_tag, regex_tag.lstrip('%{').rstrip('}'))
    else:
        bufsize_item = regex_optional_tags.sub('', bufsize_item)

    return bufsize_item

def substitute_colors(text):
    if int(version) >= 0x00040200:
        return weechat.string_eval_expression(text,{},{},{})
    # substitute colors in output
    return re.sub(regex_color, lambda match: weechat.color(match.group(1)), text)

def count_lines(ptr_window,ptr_buffer):
    global filter_status

    hdata_buf = weechat.hdata_get('buffer')
    hdata_lines = weechat.hdata_get('lines')
    lines = weechat.hdata_pointer(hdata_buf, ptr_buffer, 'lines') # own_lines, mixed_lines
    lines_count = weechat.hdata_integer(hdata_lines, lines, 'lines_count')

    hdata_window = weechat.hdata_get('window')
    hdata_winscroll = weechat.hdata_get('window_scroll')
    window_scroll = weechat.hdata_pointer(hdata_window, ptr_window, 'scroll')
    lines_after = weechat.hdata_integer(hdata_winscroll, window_scroll, 'lines_after')
    window_height = weechat.window_get_integer(weechat.current_window(), 'win_chat_height')

    filtered = 0
    filtered_before = 0
    filtered_after = 0
    # if filter is disabled, don't count.
    if (OPTIONS['count_filtered_lines'].lower() == 'off') and filter_status == 1:
        filtered, filtered_before,filtered_after = count_filtered_lines(ptr_buffer,lines_count,lines_after)
        lines_count = lines_count - filtered
#        lines_after = lines_after - filtered_after

    if lines_count > window_height:
        differential = lines_count - window_height
        percent = max(int(round(100. * (differential - lines_after) / differential)), 0)
    else:
        percent = 100

    # get current position
    current_line = lines_count - lines_after

    return lines_after,lines_count,percent,current_line, filtered, filtered_before, filtered_after

def count_filtered_lines(ptr_buffer,lines_count,lines_after):
    filtered_before = 0
    filtered_after = 0
    filtered = 0

    lines = weechat.hdata_pointer(weechat.hdata_get('buffer'), ptr_buffer, 'own_lines')
    counter = 0
    current_position = lines_count - lines_after

    if lines:
        line = weechat.hdata_pointer(weechat.hdata_get('lines'), lines, 'first_line')
        hdata_line = weechat.hdata_get('line')
        hdata_line_data = weechat.hdata_get('line_data')

        while line:
            data = weechat.hdata_pointer(hdata_line, line, 'data')
            if data:
#                message = weechat.hdata_string(hdata_line_data, data, 'message')
                displayed = weechat.hdata_char(hdata_line_data, data, 'displayed')
                if displayed == 0:
#                    weechat.prnt('','%d - %s - %s' % (counter, displayed, message))
                    if counter < current_position:
                        filtered_before += 1
                    else:
                        filtered_after += 1
            counter += 1
            line = weechat.hdata_move(hdata_line, line, 1)

    filtered = filtered_before + filtered_after
    return filtered,filtered_before,filtered_after

def update_cb(data, signal, signal_data):
    weechat.bar_item_update(SCRIPT_NAME)
    return weechat.WEECHAT_RC_OK

def filtered_update_cb(data, signal, signal_data):
    global filter_status
    if signal == 'filters_disabled':
        filter_status = 0
    if signal == 'filters_enabled':
        filter_status = 1
    weechat.bar_item_update(SCRIPT_NAME)
    return weechat.WEECHAT_RC_OK
# ================================[ weechat options and description ]===============================
def init_options():
    for option,value in OPTIONS.items():
        if not weechat.config_is_set_plugin(option):
            weechat.config_set_plugin(option, value[0])
            OPTIONS[option] = value[0]
        else:
            OPTIONS[option] = weechat.config_get_plugin(option)
        weechat.config_set_desc_plugin(option, "%s (default: '%s')" % (value[1], value[0]))

def toggle_refresh(pointer, name, value):
    global OPTIONS
    option = name[len('plugins.var.python.' + SCRIPT_NAME + '.'):]        # get optionname
    OPTIONS[option] = value                                               # save new value
    weechat.bar_item_update(SCRIPT_NAME)
    return weechat.WEECHAT_RC_OK
# ================================[ main ]===============================
if __name__ == "__main__":
#    global filter_status
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', ''):
        version = weechat.info_get("version_number", "") or 0

        if int(version) >= 0x00030600:
            filter_status = int(weechat.info_get('filters_enabled',''))
            bar_item = weechat.bar_item_new(SCRIPT_NAME, 'show_item','')
            weechat.bar_item_update(SCRIPT_NAME)
            weechat.hook_signal('buffer_line_added','update_cb','')
            weechat.hook_signal('window_scrolled','update_cb','')
            weechat.hook_signal('buffer_switch','update_cb','')
            weechat.hook_signal('*filters*','filtered_update_cb','')
            weechat.hook_command_run('/buffer clear*','update_cb','')
            weechat.hook_command_run('/window page*','update_cb','')
            weechat.hook_command_run('/input zoom_merged_buffer','update_cb','')
            weechat.hook_config( 'plugins.var.python.' + SCRIPT_NAME + '.*', 'toggle_refresh', '' )
            init_options()
        else:
            weechat.prnt('','%s%s %s' % (weechat.prefix('error'),SCRIPT_NAME,': needs version 0.3.6 or higher'))
